@@ -12,8 +12,10 @@
 #' @param PairScorer function taking four arguments, `splits1`, `splits2`,
 #' `nSplits1`, `nSplits2`, which should return the score of each pair of splits
 #' in a two-dimensional matrix.  Additional parameters may be specified via 
-#' \dots.
-#' @param \dots Additional parameters to `PairScorer`
+#' \code{...}.
+#' @param maximize Logical specifying whether the optimal matching maximizes
+#' or minimizes the scores obtained by `PairScorer()`.
+#' @param \dots Additional parameters to `PairScorer()`.
 #' 
 #' @return A numeric value specifying the score of the tree pairs under the 
 #' specified pair scorer. If `reportMatching = TRUE`, attribute also list:
@@ -62,6 +64,7 @@ GeneralizedRF <- function(splits1, splits2, nTip, PairScorer,
       matched2 <- matching[matched1]
       matched1 <- which(matched1)
       
+      # Textual representation of matchings
       attr(ret, "matchedSplits") <- 
         ReportMatching(splits1[[matched1]],
                        splits2[[matched2]],
@@ -70,14 +73,12 @@ GeneralizedRF <- function(splits1, splits2, nTip, PairScorer,
                        } else rep(TRUE, length(matched1)))
     }
     
-    attr(ret, "matchedScores") <- vapply(
-      seq_along(matching),
-      function(i) pairScores[i, matching[[i]]],
-      vector(mode(pairScores), 1)
-    )
+    attr(ret, "matchedScores") <- pairScores[
+      matrix(c(seq_along(matching), matching), ncol = 2L)]
     
     attr(ret, "pairScores") <- pairScores
   }
+  
   # Return:
   ret
 }
