@@ -236,6 +236,9 @@ IntegerVector robinson_foulds_all_pairs(List tables) {
       
       ClusterTable* Tj = tbl[j];
       
+      // Reset stack pointer for each tree pair comparison
+      S_top = S_entries.data();
+      
       Tj->TRESET();
       Tj->NVERTEX_short(&v, &w);
       
