@@ -15,84 +15,61 @@
  R. Jonker and A. Volgenant, University of Amsterdam.
  
  *
+ CHANGED 2025-08-01 by Martin Smith <martin.smith@durham.ac.uk> to optimize
+ performance.
  CHANGED 2020-01-01 by Martin Smith <martin.smith@durham.ac.uk> for 
  integration with R.
  CHANGED 2016-05-13 by Yong Yang(yongyanglink@gmail.com) in column reduction 
- part according to matlab version of LAPJV algorithm 
- https://github.com/yongyanghz/LAPJV-algorithm-c/blob/master/LAPJV/lap.cpp
+ part according to matlab version of LAPJV algorithm
+ https://github.com/yongyanghz/LAPJV-algorithm-c/blob/master/src/lap.cpp
  (Copyright (c) 2010, Yi Cao All rights reserved)--
  https://www.mathworks.com/matlabcentral/fileexchange/26836-lapjv-jonker-volgenant-algorithm-for-linear-assignment-problem-v3-0:
  *
  *************************************************************************/
 #include "tree_distances.h"
-using namespace Rcpp;
-using namespace std;
-
+#include <Rcpp/Lightest>
 
 // [[Rcpp::export]]
-List lapjv (NumericMatrix x, NumericVector maxX) {
-  const int16 n_row = x.nrow(), n_col = x.ncol(),
-              max_dim = (n_row > n_col) ? n_row : n_col,
-              spare_rows = n_row - n_col
-              ;
-  const cost max_score = BIG / max_dim;
+Rcpp::List lapjv(Rcpp::NumericMatrix &x, Rcpp::NumericVector &maxX) {
+  const lap_dim n_row = x.nrow();
+  const lap_dim n_col = x.ncol();
+  const lap_dim max_dim = std::max(n_row, n_col);
+  const lap_dim spare_rows = n_row - n_col;
+  const cost max_score = cost(BIG / max_dim);
   const double x_max = maxX[0];
   
-  lap_col *rowsol = new lap_col[max_dim];
-  lap_row *colsol = new lap_row[max_dim];
-  cost *u = new cost[max_dim], *v = new cost[max_dim];
+  std::vector<lap_col> rowsol(max_dim);
+  std::vector<lap_row> colsol(max_dim);
   
-  cost** input = new cost*[max_dim];
-  for (int16 i = 0; i != max_dim; i++) {
-    input[i] = new cost[max_dim];
+  cost_matrix input(x, x_max);
+  
+  cost score = lap(max_dim, input, rowsol, colsol);
+  
+  std::vector<int> matching;
+  matching.reserve(n_row);
+  
+  for (lap_dim i = 0; i < n_row; ++i) {
+    const int match = (rowsol[i] < n_col) ? rowsol[i] + 1 : NA_INTEGER;
+    matching.push_back(match);
   }
   
-  for (int16 r = n_row; r--;) {
-    for (int16 c = n_col; c--;) {
-      input[r][c] = cost(max_score * (x(r, c) / x_max));
-    }
-    for (int16 c = n_col; c != max_dim; c++) {
-      input[r][c] = max_score;
-    }
-  }
-  for (int16 r = n_row; r != max_dim; r++) {
-    for (int16 c = 0; c != max_dim; c++) {
-      input[r][c] = max_score;
-    }
-  }
-  
-  cost score = lap(max_dim, input, rowsol, colsol, u, v);
-  IntegerVector matching (n_row);
-  for (int16 i = 0; i != n_row; i++) {
-    matching[i] = rowsol[i] < n_col ? rowsol[i] + 1 : NA_INTEGER;
-  }
-  
-  delete [] u;
-  delete [] v;
-  delete [] rowsol;
-  delete [] colsol;
-  for (int16 i = 0; i != max_dim; i++) delete [] input[i];
-  delete [] input;
-  
-  return List::create(
-    Named("score") = (double(score) - (std::abs(spare_rows) * max_score))
-    / max_score * x_max,
-    _["matching"] = matching
+  return Rcpp::List::create(
+    Rcpp::Named("score") = (static_cast<double>(score) -
+      (std::abs(spare_rows) * max_score)) / max_score * x_max,
+    Rcpp::_["matching"] = matching
   );
 }
 
-bool nontrivially_less_than(cost a, cost b) {
+inline bool nontrivially_less_than(cost a, cost b) noexcept {
   return a + ((a > ROUND_PRECISION) ? 8 : 0) < b;
 }
 
 /* This function is the jv shortest augmenting path algorithm to solve the 
    assignment problem */
-cost lap(int16 dim,
-         cost **input_cost,
-         lap_col *rowsol,
-         lap_row *colsol,
-         cost *u,
-         cost *v)
+cost lap(const lap_row dim,
+         cost_matrix &input_cost,
+         std::vector<lap_col> &rowsol,
+         std::vector<lap_row> &colsol)
   
   // input:
   // dim        - problem size
@@ -101,47 +78,26 @@ cost lap(int16 dim,
   // output:
   // rowsol     - column assigned to row in solution
   // colsol     - row assigned to column in solution
-  // u          - dual variables, row reduction numbers
-  // v          - dual variables, column reduction numbers
   
 {
-  bool unassignedfound;
-  lap_row  i, imin, num_free = 0, previous_num_free, f, i0, k, free_row,
-           *predecessor, *freeunassigned;
-  lap_col  j, j1, j2 = 0, endofpath = 0, last = 0, low, up, *col_list, *matches;
-  /* Initializing min, endofpath, j2 and last is unnecessary, 
-   * but avoids compiler warnings */
-  cost min = 0, h, umin, usubmin, v2, *d;
-  
-  freeunassigned = new lap_row[dim];        // List of unassigned rows.
-  col_list = new lap_col[dim];    // List of columns to be scanned in various ways.
-  matches = new lap_col[dim];     // Counts how many times a row could be assigned.
-  d = new cost[dim];              // 'Cost-distance' in augmenting path calculation.
-  predecessor = new lap_row[dim]; // Row-predecessor of column in augmenting/alternating path.
-  
-  // Init how many times a row will be assigned in the column reduction.
-  for (i = 0; i != dim; i++) {
-    matches[i] = 0;
-  }
+  lap_row num_free = 0;
+  alignas(64) std::vector<cost> v(((dim + BLOCK_SIZE - 1) / BLOCK_SIZE) * BLOCK_SIZE);
+  const cost* __restrict__ v_ptr = v.data();
+  std::vector<lap_col> matches(dim); // Counts how many times a row could be assigned.
   
   // COLUMN REDUCTION
-  for (j = dim; j--; ) { // Reverse order gives better results.
-    // Find minimum cost over rows.
-    min = input_cost[0][j];
-    imin = 0;
-    for (i = 1; i != dim; i++) {
-      if (input_cost[i][j] < min) {
-        min = input_cost[i][j];
-        imin = i;
-      }
-    }
+  for (lap_col j = dim; j--; ) { // Reverse order gives better results.
+    
+    const auto [min, imin] = input_cost.findColMin(j);
     v[j] = min;
-    if (++matches[imin] == 1) {
+    ++matches[imin];
+    
+    if (matches[imin] == 1) {
       // Init assignment if minimum row assigned for first time.
       rowsol[imin] = j;
       colsol[j] = imin;
-    } else if (v[j] < v[rowsol[imin]]) {
-      int16 j1 = rowsol[imin];
+    } else if (v_ptr[j] < v_ptr[rowsol[imin]]) {
+      const lap_col j1 = rowsol[imin];
       rowsol[imin] = j;
       colsol[j] = imin;
       colsol[j1] = -1;
@@ -151,65 +107,68 @@ cost lap(int16 dim,
   }
   
   // REDUCTION TRANSFER
-  for (i = 0; i != dim; i++) {
-    if (matches[i] == 0) { // Fill list of unassigned 'free' rows.
+  std::vector<lap_row> freeunassigned(dim);        // List of unassigned rows.
+  
+  for (lap_row i = 0; i < dim; ++i) {
+    if (matches[i] == 0) {
+      // Fill list of unassigned 'free' rows.
       freeunassigned[num_free++] = i;
-    } else {
-      if (matches[i] == 1) { // Transfer reduction from rows that are assigned once.
-        j1 = rowsol[i];
-        min = BIG;
-        for (j = 0; j < dim; j++) {
-          if (j != j1) {
-            if (input_cost[i][j] - v[j] < min) {
-              min = input_cost[i][j] - v[j];
-            }
+    } else if (matches[i] == 1) {
+      // Transfer reduction from rows that are assigned once.
+      const lap_col j1 = rowsol[i];
+      const cost* row_i = input_cost.row(i);
+      cost min_cost;
+      if (j1 == 0) {
+        // Just worth the trouble to initialize with a realistic value
+        min_cost = row_i[1] - v_ptr[1];
+        
+        for (lap_col j = 2; j < dim; ++j) {
+          const cost reduced_cost = row_i[j] - v_ptr[j];
+          if (reduced_cost < min_cost) {
+            min_cost = reduced_cost;
+          }
+        }
+      } else {
+        min_cost = row_i[0] - v_ptr[0];
+        
+        for (lap_col j = 1; j < dim; ++j) {
+          if (j == j1) continue;
+          
+          const cost reduced_cost = row_i[j] - v_ptr[j];
+          if (reduced_cost < min_cost) {
+            min_cost = reduced_cost;
           }
         }
-        v[j1] = v[j1] - min;
       }
+      v[j1] -= min_cost;
     }
   }
   
   //   AUGMENTING ROW REDUCTION
-  int16 loopcnt = 0;           // do-loop to be done twice.
+  std::vector<lap_col> col_list(dim);    // List of columns to be scanned in various ways.
+  int loopcnt = 0;                       // do-loop to be done twice.
+  
   do {
-    loopcnt++;
+    ++loopcnt;
     
     //     Scan all free rows.
     //     In some cases, a free row may be replaced with another one to be 
     //     scanned next.
-    k = 0;
-    previous_num_free = num_free;
+    lap_row previous_num_free = num_free;
     num_free = 0;             // Start list of rows still free after augmenting
                               // row reduction.
+    lap_row k = 0;
     while (k < previous_num_free) {
-      i = freeunassigned[k];
-      k++;
+      //  Find minimum and second minimum reduced cost over columns.
+      const lap_row i = freeunassigned[k++];
+      const auto [umin, usubmin, min_idx, j2] = input_cost.findRowSubmin(&i, v);
+      lap_col j1 = min_idx;
       
-      //     Find minimum and second minimum reduced cost over columns.
-      umin = input_cost[i][0] - v[0];
-      j1 = 0;
-      usubmin = BIG;
-      for (j = 1; j < dim; j++) {
-        h = input_cost[i][j] - v[j];
-        if (h < usubmin) {
-          if (h >= umin) {
-            usubmin = h;
-            j2 = j;
-          } else {
-            usubmin = umin;
-            umin = h;
-            j2 = j1;
-            j1 = j;
-          }
-        }
-      }
-      
-      i0 = colsol[j1];
+      lap_row i0 = colsol[j1];
       if (nontrivially_less_than(umin, usubmin)) {
         //  Change the reduction of the minimum column to increase the minimum
         //  reduced cost in the row to the subminimum.
-        v[j1] = v[j1] - (usubmin - umin);
+        v[j1] -= (usubmin - umin);
       } else if (i0 > -1) {
           // Minimum and subminimum equal.
           // Minimum column j1 is assigned.
@@ -238,22 +197,32 @@ cost lap(int16 dim,
   } while (loopcnt < 2); // Repeat once.
   
   // AUGMENT SOLUTION for each free row.
-  for (f = 0; f != num_free; f++) {
-    free_row = freeunassigned[f];       // Start row of augmenting path.
+  std::vector<cost> d(dim);              // 'Cost-distance' in augmenting path calculation.
+  std::vector<lap_row> predecessor(dim); // Row-predecessor of column in augmenting/alternating path.
+  
+  for (lap_row f = 0; f < num_free; ++f) {
+    bool unassignedfound = false;
+    lap_row free_row = freeunassigned[f];       // Start row of augmenting path.
+    const cost* free_row_cost = input_cost.row(free_row);
+    lap_col endofpath = 0;
+    lap_col last = 0;
+    lap_row i;
+    lap_col j1;
     
     // Dijkstra shortest path algorithm.
     // Runs until unassigned column added to shortest path tree.
-    for(j = dim; j--;) {
-      d[j] = input_cost[free_row][j] - v[j];
+    for (lap_col j = 0; j < dim; ++j) {
+      d[j] = free_row_cost[j] - v_ptr[j];
       predecessor[j] = free_row;
       col_list[j] = j;        // Init column list.
     }
     
-    low = 0; // Columns in 0..low-1 are ready, now none.
-    up = 0;  // Columns in low..up-1 are to be scanned for current minimum, now none.
+    cost min = 0;
+    lap_col low = 0; // Columns in 0..low-1 are ready, now none.
+    lap_col up = 0;  // Columns in low..up-1 are to be scanned for current minimum, now none.
     // Columns in up..dim-1 are to be considered later to find new minimum;
     // at this stage the list simply contains all columns.
-    unassignedfound = false;
+    
     do {
       if (up == low) { // No more columns to be scanned for current minimum.
         last = low - 1;
@@ -261,9 +230,10 @@ cost lap(int16 dim,
         // Scan columns for up..dim-1 to find all indices for which new minimum occurs.
         // Store these indices between low..up-1 (increasing up).
         min = d[col_list[up++]];
-        for (k = up; k < dim; k++) {
-          j = col_list[k];
-          h = d[j];
+        
+        for (lap_dim k = up; k < dim; ++k) {
+          const lap_col j = col_list[k];
+          const cost h = d[j];
           if (h <= min) {
             if (h < min) {   // New minimum.
               up = low;      // Restart list at index low.
@@ -276,7 +246,7 @@ cost lap(int16 dim,
         }
         // Check if any of the minimum columns happens to be unassigned.
         // If so, we have an augmenting path right away.
-        for (k = low; k != up; k++) {
+        for (lap_dim k = low; k < up; ++k) {
           if (colsol[col_list[k]] < 0) {
             endofpath = col_list[k];
             unassignedfound = true;
@@ -288,14 +258,14 @@ cost lap(int16 dim,
       if (!unassignedfound) {
         // Update 'distances' between free_row and all unscanned columns,
         // via next scanned column.
-        j1 = col_list[low];
-        low++;
+        j1 = col_list[low++];
         i = colsol[j1];
-        h = input_cost[i][j1] - v[j1] - min;
+        const cost* row_i = input_cost.row(i);
+        const cost h = row_i[j1] - v_ptr[j1] - min;
         
-        for (k = up; k != dim; k++) {
-          j = col_list[k];
-          v2 = input_cost[i][j] - v[j] - h;
+        for (lap_dim k = up; k < dim; ++k) {
+          const lap_col j = col_list[k];
+          cost v2 = row_i[j] - v_ptr[j] - h;
           if (v2 < d[j]) {
             predecessor[j] = i;
             if (v2 == min) { // New column found at same minimum value
@@ -317,9 +287,9 @@ cost lap(int16 dim,
     } while (!unassignedfound);
     
     // Update column prices.
-    for(k = last + 1; k--;) {
+    for(lap_dim k = 0; k <= last; ++k) {
       j1 = col_list[k];
-      v[j1] = v[j1] + d[j1] - min;
+      v[j1] += d[j1] - min;
     }
     
     // Reset row and column assignments along the alternating path.
@@ -334,18 +304,9 @@ cost lap(int16 dim,
   
   // Calculate optimal cost.
   cost lapcost = 0;
-  for(i = dim; i--;) {
-    j = rowsol[i];
-    u[i] = input_cost[i][j] - v[j];
-    lapcost = lapcost + input_cost[i][j];
+  for (lap_dim i = 0; i < dim; ++i) {
+    lapcost += input_cost(i, rowsol[i]);
   }
-  
-  // Free reserved memory.
-  delete[] predecessor;
-  delete[] freeunassigned;
-  delete[] col_list;
-  delete[] matches;
-  delete[] d;
-  
+
   return lapcost;
 }
