@@ -1,4 +1,4 @@
-library('TreeTools')
+library("TreeTools")
 test_that("Median is calculated", {
   tenTrees <- as.phylo(1:10, 8L)
   expect_equal(4, median(tenTrees, index = TRUE))
@@ -15,4 +15,4 @@ test_that("Median is calculated", {
   expect_equal(as.phylo(c(3:5, 7), 8),
                median(tenTrees, Distance = RobinsonFoulds, breakTies = FALSE),
                ignore_attr = TRUE)
-})
\ No newline at end of file
+})
